// Solution: Boris Mihov

#include <algorithm>
#include <iostream>
#include <cassert>
#include <numeric>
#include <vector>

typedef long long llong;
const int MAXN = 2000 + 10;
const llong INF = 2e9 + 10;

int l[MAXN], n;
int inputC[MAXN];
int c[MAXN][MAXN];
std::pair <llong,int> dpPlus[MAXN][MAXN];
std::pair <llong,int> dpMinus[MAXN][MAXN];
bool blPlus[MAXN][MAXN];
bool blMinus[MAXN][MAXN];
std::vector <int> g[MAXN];

std::pair <llong,int> plus(int node, int incoming);
std::pair <llong,int> minus(int node, int wanted);

inline bool operator < (std::pair <llong,int> &a, const std::pair <llong,int> &b)
{
    if (a.first != b.first) return a.first < b.first;
    return a.second > b.second;
}

inline void operator += (std::pair <llong,int> &a, const std::pair <llong,int> &b)
{
    a.first += b.first;
    a.second += b.second;
}

inline std::pair <llong,int> operator + (std::pair <llong,int> &a, const std::pair <llong,int> &b)
{
    return {a.first + b.first, a.second + b.second};
}

inline std::pair <llong,int> operator - (std::pair <llong,int> &a, const std::pair <llong,int> &b)
{
    return {a.first - b.first, a.second - b.second};
}

void dfs(int node, int p)
{
    for (int &i : g[node])
    {
        if (i == p)
        {
            std::swap(i, g[node].back());
            g[node].pop_back();
            break;
        }
    }

    for (const int &i : g[node])
    {
        dfs(i, node);
    }
}

std::pair <llong,int> plus(int node, int incoming)
{
    if (incoming >= n+1) return {0, 0};
    if (g[node].empty())
    {
        assert(incoming >= 1);
        return {0, 0};
    }

    if (blPlus[node][incoming]) return dpPlus[node][incoming];
    blPlus[node][incoming] = true;
    dpPlus[node][incoming] = minus(node, 0);

    std::pair <llong,int> lsum = {0, 0};
    for (const int &i : g[node])
    {
        if (incoming == 1) lsum += minus(i, 0);
        else lsum += plus(i, incoming - 1);
    }

    dpPlus[node][incoming] = std::min(dpPlus[node][incoming], lsum);
    return dpPlus[node][incoming];
}

std::pair <llong,int> minus(int node, int wanted)
{
    if (wanted >= n+1) return {INF, 0};
    if (g[node].empty())
    {
        if (l[node] >= wanted + 1) return {c[node][wanted + 1], 1};
        return {INF, 0};
    }

    if (blMinus[node][wanted]) return dpMinus[node][wanted];
    blMinus[node][wanted] = true;
    dpMinus[node][wanted] = minus(node, wanted + 1);
    
    std::pair <llong,int> sum = {0, 0};
    for (const int &i : g[node])
    {
        if (wanted >= 1) sum += plus(i, wanted);
        else sum += minus(i, 0);
    }

    if (wanted + 1 <= l[node])
    {
        dpMinus[node][wanted] = std::min(dpMinus[node][wanted], {sum.first + c[node][wanted + 1], sum.second + 1});
    }

    for (const int &i : g[node])
    {
        std::pair <llong,int> curr = sum + minus(i, wanted + 1);
        if (wanted == 0) curr = curr - minus(i, 0);
        else curr = curr - plus(i, wanted);

        if (curr < dpMinus[node][wanted])
        {
            dpMinus[node][wanted] = curr;
        }
    }
    
    return dpMinus[node][wanted];
}

void solve()
{
    for (int i = 1 ; i <= n ; ++i)
    {
        for (int j = n + 1 ; j >= l[i] + 1 ; --j) c[i][j] = INF;
        for (int j = l[i] ; j >= 1 ; --j)
        {
            c[i][j] = std::min(c[i][j + 1], inputC[j]);
        }
    }

    dfs(1, 0);
    std::pair <llong,int> res = minus(1, 0);
    if (res.first >= INF) std::cout << -1 << '\n';
    else std::cout << res.first << ' ' << res.second << '\n';
}

void read()
{
    std::cin >> n;
    for (int i = 1 ; i <= n ; ++i) std::cin >> inputC[i];
    for (int i = 1 ; i <= n ; ++i) std::cin >> l[i];
    for (int i = 2 ; i <= n ; ++i)
    {
        int x, y;
        std::cin >> x >> y;
        g[x].push_back(y);
        g[y].push_back(x);
    }
    
}

void fastIO()
{
    std::ios_base :: sync_with_stdio(0);
    std::cout.tie(nullptr);
    std::cin.tie(nullptr);
}

int main()
{
    fastIO();
    read();
    solve();

    return 0;
}