// End me
// Scroll trough it at your own risk

#include "testlib.h"
#include <algorithm>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <numeric>
#include <cstring>
#include <cassert>
#include <vector>
#include <queue>
#include <cmath>

typedef long long llong;
const llong MAXN = 2000 + 10;
const llong SUBTASKS = 10;
const llong INF  = 1e9;

llong c[MAXN];
llong l[MAXN], n;
llong cost[MAXN][MAXN];
std::pair <llong,llong> dpPlus[MAXN][MAXN];
std::pair <llong,llong> dpMinus[MAXN][MAXN];
bool blPlus[MAXN][MAXN];
bool blMinus[MAXN][MAXN];
std::vector <llong> g[MAXN];

std::pair <llong,llong> plus(llong node, llong incoming);
std::pair <llong,llong> minus(llong node, llong wanted);

inline bool operator < (std::pair <llong,llong> &a, const std::pair <llong,llong> &b)
{
    if (a.first != b.first) return a.first < b.first;
    return a.second < b.second;
}

inline void operator += (std::pair <llong,llong> &a, const std::pair <llong,llong> &b)
{
    a.first += b.first;
    a.second += b.second;
}

inline std::pair <llong,llong> operator + (std::pair <llong,llong> &a, const std::pair <llong,llong> &b)
{
    return {a.first + b.first, a.second + b.second};
}

inline std::pair <llong,llong> operator - (std::pair <llong,llong> &a, const std::pair <llong,llong> &b)
{
    return {a.first - b.first, a.second - b.second};
}

void dfs(llong node, llong p)
{
    // std::cout << "dfs: " << node << ' ' << p << '\n' << std::flush;
    for (llong &i : g[node])
    {
        if (i == p)
        {
            std::swap(i, g[node].back());
            g[node].pop_back();
            break;
        }
    }

    for (const llong &i : g[node])
    {
        dfs(i, node);
    }
}

std::pair <llong,llong> plus(llong node, llong incoming)
{
    // std::cout << "plus: " << node << ' ' << incoming << '\n' << std::flush;
    if (incoming >= n+1) return {0, 0};
    if (g[node].empty())
    {
        assert(incoming >= 1);
        return {0, 0};
    }

    if (blPlus[node][incoming]) return dpPlus[node][incoming];
    blPlus[node][incoming] = true;
    dpPlus[node][incoming] = minus(node, 0);

    std::pair <llong,llong> lsum = {0, 0};
    for (const llong &i : g[node])
    {
        if (incoming == 1) lsum += minus(i, 0);
        else lsum += plus(i, incoming - 1);
    }

    dpPlus[node][incoming] = std::min(dpPlus[node][incoming], lsum);
    return dpPlus[node][incoming];
}

std::pair <llong,llong> minus(llong node, llong wanted)
{
    // std::cout << "minus: " << node << ' ' << wanted << '\n' << std::flush;
    if (wanted >= n+1) return {INF, 0};
    if (g[node].empty())
    {
        if (l[node] >= wanted + 1) return {cost[node][wanted + 1], 1};
        return {INF, 0};
    }

    if (blMinus[node][wanted]) return dpMinus[node][wanted];
    blMinus[node][wanted] = true;
    dpMinus[node][wanted] = minus(node, wanted + 1);
    
    std::pair <llong,llong> sum = {0, 0};
    for (const llong &i : g[node])
    {
        if (wanted >= 1) sum += plus(i, wanted);
        else sum += minus(i, 0);
    }

    if (wanted + 1 <= l[node])
    {
        dpMinus[node][wanted] = std::min(dpMinus[node][wanted], {sum.first + cost[node][wanted + 1], sum.second + 1});
    }

    for (const llong &i : g[node])
    {
        std::pair <llong,llong> curr = sum + minus(i, wanted + 1);
        if (wanted == 0) curr = curr - minus(i, 0);
        else curr = curr - plus(i, wanted);

        if (curr < dpMinus[node][wanted])
        {
            dpMinus[node][wanted] = curr;
        }
    }
    
    return dpMinus[node][wanted];
}

llong getBombs()
{
    for (llong i = 1 ; i <= n ; ++i)
    {
        for (int j = 0 ; j <= n + 1 ; ++j)
        {
            dpPlus[i][j] = {0, 0};
            blPlus[i][j] = 0;
            dpMinus[i][j] = {0, 0};
            blMinus[i][j] = 0;
        }
        for (llong j = n + 1 ; j >= l[i] + 1 ; --j) cost[i][j] = INF;
        for (llong j = l[i] ; j >= 1 ; --j)
        {
            cost[i][j] = std::min(cost[i][j + 1], c[j]);
        }
    }

    dfs(1, 0);
    // std::cout << "done dfs\n";
    std::pair <llong,llong> res = minus(1, 0);
    // std::cout << "dp res: " << res.first << ' ' << res.second << '\n';
    if (res.first >= INF) return 0;
    return res.second;
}

inline std::string getFilename(llong num)
{
    std::string first; first += num / 10 + '0';
    std::string second; second += num % 10 + '0';
    return "garden." + first + second + ".in"; 
}

std::pair <llong,llong> edge[MAXN];
bool doSubtask[SUBTASKS];
llong leftRange[SUBTASKS] = {0, 1, 3, 12, 21, 30, 39, 48, 57};
llong rightRange[SUBTASKS] = {0, 2, 11, 20, 29, 38, 47, 56, 65};
bool isStick[SUBTASKS] = {0, 0, 0, 1, 1, 1, 0, 0, 0};
llong constraints[SUBTASKS] = {0, 0, 8, 75, 500, 2000, 75, 500, 2000};
double exponent[SUBTASKS][9];
double add[SUBTASKS][9];
llong currT[SUBTASKS][9];

llong bombsR[SUBTASKS][9];
llong bombsL[SUBTASKS][9];

llong smallPercentage[9] = {25, 100, 50, 60, 40, 100, -1, 60};
llong mediumPercentage[9] = {25, 0, 50, 30, 30, 0, -1, 30};
llong bigPercentage[9] = {50, 0, 0, 10, 30, 0, -1, 10};
llong hugeCnt[9] = {0, 1, 0, 0, 0, 6, 0, 0};
bool typeTest[9] = {0, 1, 1, 1, 1, 0, 0, 0};

llong perm[MAXN];
void buildTree(llong subtask, llong test)
{
    n = constraints[subtask];
    for (llong i = 1 ; i <= n ; ++i)
    {
        g[i].clear();
    }

    std::iota(perm + 1, perm + 1 + n, 1);
    shuffle(perm + 1, perm + 1 + n);

    if (isStick[subtask])
    {
        for (llong i = 1 ; i <= n - 1 ; ++i)
        {
            g[perm[i]].push_back(perm[i + 1]);
            g[perm[i + 1]].push_back(perm[i]);
            edge[i] = {perm[i], perm[i + 1]};
        }
    } else
    {
        for (llong i = 2 ; i <= n ; ++i)
        {
            llong curr = rnd.wnext(i - 1, currT[subtask][test]) + 1;
            g[perm[i]].push_back(perm[curr]);
            g[perm[curr]].push_back(perm[i]);
            edge[i - 1] = {perm[i], perm[curr]};
        }
    }
}

llong dist[MAXN];
llong sumReached[MAXN];
std::queue <llong> q;
void bfs(llong node)
{
    std::fill(dist + 1, dist + 1 + n, 0);
    dist[node] = 1;
    q.push(node);

    llong currCnt = 0;
    while (!q.empty())
    {
        currCnt++;
        llong top = q.front();
        q.pop();

        for (const llong &i : g[top])
        {
            if (dist[i] == 0)
            {
                dist[i] = dist[top] + 1;
                q.push(i);
            }
        }

        if (q.empty() || dist[q.front()] > dist[top])
        {
            c[dist[top]] += currCnt;
            if (q.empty())
            {
                for (llong i = dist[top] + 1 ; i <= n ; ++i)
                {
                    c[i] += currCnt;
                }
            }
        }
    }
}

inline llong getSmall(llong n)
{
    llong leftBorder = 0;
    llong rightBorder = cbrt(n);
    return rnd.next(rightBorder - leftBorder + 1) + leftBorder;
}

inline llong getMeduim(llong n)
{
    llong leftBorder = cbrt(n) + 1;
    llong rightBorder = sqrt(n) + 1;
    return rnd.next(rightBorder - leftBorder + 1) + leftBorder;
}

inline llong getBig(llong n)
{
    llong leftBorder = sqrt(n);
    llong rightBorder = cbrt(n) * cbrt(n);
    return rnd.next(rightBorder - leftBorder + 1) + leftBorder;
}

inline llong getHuge(llong n)
{
    llong leftBorder = std::min(n/2, (llong)(cbrt(n) * cbrt(n) + 1));
    llong rightBorder = n/2;
    return rnd.next(rightBorder - leftBorder + 1) + leftBorder;
}

void buildTest(llong subtask, llong number)
{
    n = constraints[subtask];
    std::fill(l + 1, l + 1 + n, 0);
    std::fill(c + 1, c + 1 + n, 0);
    if (number != 7 && number != 8)
    {
        for (llong i = 1 ; i <= n ; ++i)
        {
            bfs(i);
        }

        for (llong i = 1 ; i <= n ; ++i)
        {
            c[i] /= n;
            // std::cout << "i: " << i << ' ' << std::fixed << std::setprecision(10) << ' ' << exponent[subtask][number] << ' ' << c[i] << ' ' << pow(exponent[subtask][number], c[i]) << '\n';
            if (!typeTest[number]) c[i] = ((double)add[subtask][number] * c[i] * (1 + cbrt(cbrt(i)))) * pow(exponent[subtask][number], c[i]);
            else c[i] = ((double)add[subtask][number] * c[i] * (1 + cbrt(cbrt(i)))) / pow(exponent[subtask][number], c[i]);
            // std::cout << "after: " << c[i] << '\n';
        }
    } else
    {
        for (llong i = 1 ; i <= n ; ++i)
        {
            c[i] = rnd.next((llong)1e6);
        }

        std::sort(c + 1, c + 1 + n);
    }

    if (number != 6 && number != 8)
    {
        llong ptr = 1;
        for (llong i = 1 ; i <= hugeCnt[number] ; ++i)
        {
            l[ptr++] = getHuge(n); 
        }

        for (llong i = 1 ; i <= bigPercentage[number] * n / 100 && ptr <= n ; ++i)
        {
            l[ptr++] = getBig(n);
        }

        for (llong i = 1 ; i <= mediumPercentage[number] * n / 100 && ptr <= n ; ++i)
        {
            l[ptr++] = getMeduim(n);
        }

        while (ptr <= n)
        {
            l[ptr++] = getSmall(n);
        }

        shuffle(l + 1, l + 1 + n);
    } else
    {
        for (llong i = 1 ; i <= n ; ++i)
        {
            l[i] = rnd.next(n) + 1;
        }    
    }
}

void printTest(std::string filename)
{
    shuffle(edge + 1, edge + n);
    std::ofstream fout(filename);

    fout << n << '\n';
    for (llong i = 1 ; i <= n ; ++i)
    {
        fout << c[i];
        if (i != n) fout << ' ';
    }

    fout << '\n';

    for (llong i = 1 ; i <= n ; ++i)
    {
        fout << l[i];
        if (i != n) fout << ' ';
    }

    fout << '\n';
    for (llong i = 1 ; i <= n - 1 ; ++i)
    {
        if (rnd.next(2) == 0) std::swap(edge[i].first, edge[i].second);
        fout << edge[i].first << ' ' << edge[i].second << '\n';
    }

    fout.close();
}

inline bool bigC()
{
    for (llong i = 1 ; i <= n ; ++i)
    {
        if (c[i] > (llong)1e6) return true;
    }

    return false;
}

inline bool badBombs(llong subtask, llong number)
{
    if (subtask == 2) return false;
    llong cnt = getBombs();
    return cnt < bombsL[subtask][number] || cnt > bombsR[subtask][number];
}

int main(int argc , char **argv)
{
    // cancer incoming
    bombsL[3][0] = 10;
    bombsR[3][0] = 20;
    bombsL[3][1] = 3;
    bombsR[3][1] = 5;
    bombsL[3][2] = 8;
    bombsR[3][2] = 21;
    bombsL[3][3] = 4;
    bombsR[3][3] = 10;
    bombsL[3][4] = 3;
    bombsR[3][4] = 6;
    bombsL[3][5] = 6;
    bombsR[3][5] = 8;
    bombsL[3][6] = 20;
    bombsR[3][6] = 40;
    bombsL[3][7] = 2;
    bombsR[3][7] = 5;
    bombsL[3][8] = 0;
    bombsR[3][8] = 0;

    bombsL[4][0] = 60;
    bombsR[4][0] = 165;
    bombsL[4][1] = 3;
    bombsR[4][1] = 10;
    bombsL[4][2] = 10;
    bombsR[4][2] = 175;
    bombsL[4][3] = 30;
    bombsR[4][3] = 50;
    bombsL[4][4] = 10;
    bombsR[4][4] = 20;
    bombsL[4][5] = 6;
    bombsR[4][5] = 14;
    bombsL[4][6] = 100;
    bombsR[4][6] = 300;
    bombsL[4][7] = 25;
    bombsR[4][7] = 35;
    bombsL[4][8] = 0;
    bombsR[4][8] = 0;

    bombsL[5][0] = 300;
    bombsR[5][0] = 680;
    bombsL[5][1] = 3;
    bombsR[5][1] = 20;
    bombsL[5][2] = 70;
    bombsR[5][2] = 320;
    bombsL[5][3] = 120;
    bombsR[5][3] = 200;
    bombsL[5][4] = 40;
    bombsR[5][4] = 160;
    bombsL[5][5] = 6;
    bombsR[5][5] = 20;
    bombsL[5][6] = 400;
    bombsR[5][6] = 1200;
    bombsL[5][7] = 100;
    bombsR[5][7] = 150;
    bombsL[5][8] = 0;
    bombsR[5][8] = 0;

    bombsL[6][0] = 10;
    bombsR[6][0] = 20;
    bombsL[6][1] = 3;
    bombsR[6][1] = 5;
    bombsL[6][2] = 8;
    bombsR[6][2] = 15;
    bombsL[6][3] = 4;
    bombsR[6][3] = 10;
    bombsL[6][4] = 3;
    bombsR[6][4] = 6;
    bombsL[6][5] = 6;
    bombsR[6][5] = 8;
    bombsL[6][6] = 20;
    bombsR[6][6] = 40;
    bombsL[6][7] = 2;
    bombsR[6][7] = 5;
    bombsL[6][8] = 0;
    bombsR[6][8] = 0;

    bombsL[7][0] = 60;
    bombsR[7][0] = 120;
    bombsL[7][1] = 3;
    bombsR[7][1] = 10;
    bombsL[7][2] = 40;
    bombsR[7][2] = 80;
    bombsL[7][3] = 30;
    bombsR[7][3] = 50;
    bombsL[7][4] = 10;
    bombsR[7][4] = 20;
    bombsL[7][5] = 6;
    bombsR[7][5] = 14;
    bombsL[7][6] = 100;
    bombsR[7][6] = 300;
    bombsL[7][7] = 25;
    bombsR[7][7] = 35;
    bombsL[7][8] = 0;
    bombsR[7][8] = 0;

    bombsL[8][0] = 300;
    bombsR[8][0] = 600;
    bombsL[8][1] = 3;
    bombsR[8][1] = 20;
    bombsL[8][2] = 160;
    bombsR[8][2] = 320;
    bombsL[8][3] = 120;
    bombsR[8][3] = 200;
    bombsL[8][4] = 40;
    bombsR[8][4] = 160;
    bombsL[8][5] = 6;
    bombsR[8][5] = 20;
    bombsL[8][6] = 400;
    bombsR[8][6] = 1200;
    bombsL[8][7] = 1;
    bombsR[8][7] = 1;
    bombsL[8][8] = 0;
    bombsR[8][8] = 0;
    //cancer away

    registerGen(argc, argv, 1);
    llong i;
    while (true)
    {
        std::cout << "subtask number\n";
        std::cin >> i;
        llong j;
        bool repeat = false;
        while (true)
        {
            if (!repeat)
            {
                std::cout << "test number: " << i << ' ' << constraints[i] << "?\n";
                std::cin >> j;
            }
            if (j == -1) break;
            if (j > 8) break;
            if (!isStick[j])
            {
                // std::cout << "t for: " << i << ' ' << j << "?\n";
                // std::cin >> currT[i][j];
                do currT[i][j] = constraints[i]/5 - rnd.next(2*(constraints[i]/5)+1);
                while (currT[i][j] == 0);
            }

            if (j != 7 && j != 8)
            {
                // std::cout << "exponent and add for " << i << ' ' << j << "?\n";
                // std::cin >> exponent[i][j] >> add[i][j];
                exponent[i][j] = 1.0 + rnd.next((int)1e5) / 1e7;
                add[i][j] = rnd.next(10) + 1;
                // exponent[i][j] = 1.000000008;
                // add[i][j] = rnd.next(5001) + 1;
                // std::cout << "here: " << add[i][j] << ' ' << exponent[i][j] << '\n' << std::flush;
            }

            buildTree(i, j);
            buildTest(i, j);
            printTest(getFilename(leftRange[i] + j));
            // std::cout << "done\n" << std::flush;
            if (j == 5)
            {
                llong root = cbrt(constraints[i]);
                double multBy = 1000.3;
                for (llong i = 1 ; i <= root ; ++i)
                {
                    c[i] = (double)c[i] * multBy;
                }
            }
            
            std::cout << "about to calc: " << bigC() << ' ' << getBombs() << "\n" << std::flush;
            if (bigC() || badBombs(i, j))
            {
                // std::cout << "in if\n" << std::flush;
                repeat = true;
                continue;
            } else
            {
                // std::cout << "done test: " << j << '\n' << std::flush;
                repeat = false;
            }
        }

        if (j == -1) break;
        std::cout << "done subtask: " << i << '\n';
    }

    return 0;
}